Micado-WISE
===========

This is the Micado-WISE source tree.

Overview of the directories:

* `config`: The configuration of Micado-WISE.
* `main`: The main persistent classes of the Micado-WISE data model.
* `requirements`: Persistent classes for requirements.
* `test`: Tests.
* `toolbox`: Scripts for using and managing Micado-WISE.
* `util`: Miscellaneous utilities.

Contact information Hugo Buddelmeijer, buddel@astro.rug.nl

Installation
------------
TODO: Create proper installation instructions.

For now you can:
1. follow the [Dockerfile](install/Dockerfile)
2. install a [PostgreSQL database](https://github.com/gregs1104/peg/)
3. follow [.gitlab-ci.yml](.gitlab-ci.yml)


(Data) Model
============

The Python classes in `main` can be seen as a concrete implementation of the (data) model.
The idea is that we have a one-to-one correspondence between concepts mentioned in the [design documents][1] and the code.
The ultimate goal is that this one-to-one correspondence between design and implementation is automatic.
E.g. the documents are created automatically from the code, or the code automatically from the documents, or both from a third component like XSD files.

Design Structure
----------------

The structure of the design documents:

* The [design documents][1] contain tables for each processing step, e.g. for the [Dark Current](https://docs.google.com/document/d/1QqUBEb_dv5aKi8m30sc6DQQyJqM3jGRHuO_IXBDYMUc/edit#).
* These tables are copied automatically from the [ImagingTasks spreadsheet][2], e.g. the [Dark tab](https://docs.google.com/spreadsheets/d/1t1COgxxWQso3TeUGCn9qEebM66b4ful4xYLAdsQr2Ug/edit?zx=28fb6ofggdru#gid=694297875&range=A2:B28).
* Most of the information in that spreadsheet is automatically copied from the [Data Flow Framework sheets][3], in particular the [Recipes tab](https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=1072399737), e.g. the [mcd_det_dark row](https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=1072399737&range=5:5).
* The Recipes tab is filled with information from the [DataItems tab](https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=751586500), e.g. the 'DARK_RAW' and 'MASTERDARK_IMG' rows.

Class Structure
---------------

The structure of the main Python classes:

* See [Sketchboard diagram][4] ([png](docs/classdiagram.png)).
* Each row in the [DataItems tab][3] corresponds to a class in the Python code. E.g. the `Dark` and `MasterDark` classes.
* All other information in the DataItems tab is encoded either through class inheritance ([solid arrows][4]) or through attributes, e.g.:
  * All these classes inherit from the `DataItem` class, to mark them as, well, data items described in the design.
  * '*_RAW' classes inherit from the `Raw` class.
  * Everything with DPR_TECH == IMAGE or PRO_TYPE == (MEF)IMAGE inherit from the `BaseFrame` class. (Not called 'Image' because there is a DataItem already called 'IMAGE'.)
  * (TODO: There should probably be a class for processed / DPR data items, but that does not exist.)
  * etc.
* There is a correspondence between FITS header keywords and class attributes (TODO: this doesn't actually work yet). E.g.
  * NAXIS1/NAXIS2/EXPTIME etc translate directly,
  * DATE is a simple mapping,
  * etc
* There is a (very preliminary) mapping of information in the [Recipes tab][3] to the 'DPR' classes. E.g.
  * The `MasterDark` class has an attribute `darks` that refers to the list of raw `Dark` instances that this `MasterDark` is made of ([dashed arrow][4]).
  * Such attributes are called 'dependencies', e.g. the dependencies of a `MasterDark` are (a specific set of) raw `Dark`s (and more).
* The raw data all have attributes pointing to classes modeling the instrument / telescope / atmosphere / universe:
  * The `Detector` class describes the (expected/measured/guessed) dark current of a detector (as `FPA_DARK_MEDIAN`).
  * The attributes of these classes resemble SimCADO arguments closely.
  * (TODO: There are also stubs for old 'Simulator' classes. These are currently not used and most of those will be replaced with such instrument-model classes etc.)
* Requirements are also encoded in the model directly. This can become very cool, but experimental for now.

Data Processing
---------------
How to use the code to process data:
* All processed classes have a `make()` method:
  * This creates the data from the dependencies. E.g. the `MasterDark` by median-averaging the raw Darks.
  * TODO: This should become a mapping to the recipe, e.g. `mcd_det_dark`. (Which can become a CPL plugin.)
  * If the automation works perfectly, the actual contents of these recipes will be the only thing that cannot be automatically generated.
* All raw classes have a `make_simulation()` method:
  * This will call SimCADO to create the data.
  * The input to SimCADO should all come from dependencies of the raw data, e.g. a `Detector` instance for the `FPA_DARK_MEDIAN`.
  * (TODO: Simulating processed data can also become an option, e.g. for comparison purposes.)
* All meta-data can also be stored in a database:
  * Every attribute (= also FITS header) is mapped to a database column.
  * This can be queried, and the idea is that at some point (TODO) a `MasterDark` instance can autonomously query the database for the raw `Dark`s that it needs.

[1]: https://docs.google.com/document/d/1QqUBEb_dv5aKi8m30sc6DQQyJqM3jGRHuO_IXBDYMUc/edit
[2]: https://docs.google.com/spreadsheets/d/1t1COgxxWQso3TeUGCn9qEebM66b4ful4xYLAdsQr2Ug/edit
[3]: https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit 
[4]: https://sketchboard.me/hBsizELTAfrc
