# -*- coding: utf-8 -*-
"""
The Dark represents a raw dark image.
"""

from common.database.DBMain import persistent
from micado.main.Detector import Detector
from micado.main.Raw import Raw
import simcado


class Dark(Raw):
    """
    The Dark represents a raw dark image.

    See DARK_RAW DataItem:
    https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=751586500&range=3:3
    """

    detector: Detector = persistent("Detector from which this Dark is made. [None]", Detector, None)

    def make_simulation(self):
        """
        Simulate a Dark.
        """
        # Use non_destructive mode instead of superfast mode
        # because superfast mode does not take dark current into account.
        read_out_org = simcado.Chip.read_out
        simcado.Chip.read_out = \
            lambda s, cmds, read_out_type="non_destructive": \
            read_out_org(s, cmds, read_out_type="non_destructive")
        # Run the simulation.
        sa = simcado.run(
            src=simcado.source.empty_sky(),
            INST_FILTER_TC="B",
            ATMO_USE_ATMO_BG="no",
            INST_USE_AO_MIRROR_BG="no",
            FPA_USE_NOISE="yes",
            FPA_READOUT_MEDIAN=0,
            FPA_READOUT_STDEV=0,
            FPA_DARK_MEDIAN=self.detector.FPA_DARK_MEDIAN,
            FPA_DARK_STDEV=0,
            FPA_NOISE_PATH="FPA_noise.fits",
            OBS_REMOVE_CONST_BG="no",
            FPA_LINEARITY_CURVE="none",
            OBS_EXPTIME=1.,
            filename="test.fits",
        )
        self.image = sa[0].data
        self.read_fits_header(sa[0].header)
