"""
This module defines the Imstat object.
The Imstat object is used to store image statistics.
"""

import numpy

from common.database.DBMain import DBObject, persistent
from micado.main.BaseFrame import BaseFrame


class Imstat(DBObject):
    """
    Imstat contains image statistics.

    Based on astro.main.Imstat but pointing the other way around.
    """

    baseframe: BaseFrame = persistent("The frame [None]", BaseFrame, None)

    min: float = persistent('Minimum value of data [ADU]', float, 0)
    max: float = persistent('Maximum value of data [ADU]', float, 0)
    mean: float = persistent('Mean value of data [ADU]', float, 0)
    std: float = persistent('Standard deviation of data [ADU]', float, 0)
    median: float = persistent('Median of data [ADU]', float, 0)

    def make(self):
        """
        Calculate the statistics.
        """
        self.min = float(self.baseframe.image.min())
        self.max = float(self.baseframe.image.max())
        self.mean = float(self.baseframe.image.mean())
        self.std = float(self.baseframe.image.std())
        self.median = float(numpy.median(self.baseframe.image))
