# -*- coding: utf-8 -*-
"""
The MasterDark is a dark calibration frame
"""

import numpy

from common.database.DBMain import persistent
from micado.main.BaseFrame import BaseFrame
from micado.main.Dark import Dark


class MasterDark(BaseFrame):
    """
    A master dark file.

    See MASTERDARK_IMG DataItem:
    https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=751586500&range=4:4

    See mcd_det_dark recipe:
    https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=1072399737&range=5:5
    https://docs.google.com/spreadsheets/d/1t1COgxxWQso3TeUGCn9qEebM66b4ful4xYLAdsQr2Ug/edit#gid=694297875
    """

    recipe = "mcd_det_dark"

    darks: [Dark] = persistent("A list of raw darks [None]", Dark, [])

    def make(self):
        """
        Process a MasterDark.
        """
        images = [dark.image for dark in self.darks]
        self.image = numpy.median(images, axis=0)
