# -*- coding: utf-8 -*-
"""
Requirement DPS-3000-SCI.
"""

from common.database.DBMain import persistent
from micado.main.Imstat import Imstat
from micado.main.Requirement import Requirement


class DPS3000SCI(Requirement):
    """
    Requirement DPS-3000-SCI.

    See Dark Current:
    https://docs.google.com/spreadsheets/d/1t1COgxxWQso3TeUGCn9qEebM66b4ful4xYLAdsQr2Ug/edit#gid=694297875
    """

    imstat_masterdark: Imstat = persistent("Statistics of MasterDark [None]", Imstat, None)
    dark_true: float = persistent("True value of dark_current [ADU]", float, 0.0)
    precision: float = persistent("Acceptable precision [None]", float, 0.01)

    def make(self):
        """
        Verify the condition.
        """
        result = abs(self.imstat_masterdark.median - self.dark_true) / self.dark_true < self.precision
        self.passed = int(result)
