# -*- coding: utf-8 -*-
"""
Tests for DataItem.py
"""

import hashlib
import os
import tempfile

from common.database.DataObject import DataObject
from common.database.DBSelect import Select
from micado.main.DataItem import DataItem


def test_dataitem():
    """
    Test DataItem functions.
    """
    text = "helloworld".encode('utf-8')
    md5sum = hashlib.md5(text).hexdigest()
    fname_expected = f"DataItem_{md5sum}.txt"

    ftemp = tempfile.NamedTemporaryFile(suffix=".txt", delete=False)
    fname = ftemp.name
    ftemp.write(text)
    ftemp.close()

    di1 = DataItem()
    di1.unique_filename(fname)
    assert di1.filename == fname_expected

    di2 = DataItem()
    di2.filename = fname
    di2.unique_filename(fname)
    assert di2.filename == fname_expected

    os.unlink(fname)


def create_dataitem():
    """
    Create a DataItem
    """
    di3 = DataItem()
    fn = tempfile.NamedTemporaryFile(suffix=".txt").name
    di3.filename = fn
    di3.commit()
    del di3
    return fn


def test_commit():
    """
    Test committing this.
    """
    fn = create_dataitem()
    dos: Select = (DataObject.filename == fn)
    ldos = list(dos)
    assert len(ldos) == 1, "Bad len"
    do = ldos[0]
    assert isinstance(do, DataItem), "object is not a DataItem"
