# -*- coding: utf-8 -*-
"""
Tests for MasterDark.py
"""

from common.database.Context import context
from micado.main.Detector import Detector
from micado.main.Dark import Dark
from micado.main.MasterDark import MasterDark
from micado.main.Imstat import Imstat
from micado.requirements.DPS3000SCI import DPS3000SCI


def test_dependencies():
    """
    Test setting dependencies.
    """
    dark1 = Dark()
    dark2 = Dark()
    masterdark1 = MasterDark()
    masterdark1.darks = [dark1, dark2]
    masterdark1.commit()

    oid = masterdark1.object_id.binary()
    masterdark2 = MasterDark(object_id=oid)
    dark1b, dark2b = masterdark2.darks

    assert dark1.object_id.binary() == dark1b.object_id.binary()
    assert dark2.object_id.binary() == dark2b.object_id.binary()

    context.delete(masterdark1, True)
    context.delete(dark1, True)
    context.delete(dark2, True)


def test_processing():
    """
    Test creation of MasterDark.
    """
    # Set the real dark current and create a Detector.
    dark_real = 100.
    mydetector = Detector()
    mydetector.FPA_DARK_MEDIAN = dark_real

    # Create raw Darks.
    n_darks = 5
    mydarks = []
    for i in range(n_darks):
        mydark = Dark()
        mydark.detector = mydetector
        mydark.make_simulation()
        mydarks.append(mydark)

    # Create MasterDark.
    mymasterdark = MasterDark()
    mymasterdark.darks = mydarks
    mymasterdark.make()

    # Create image statistics.
    myimstat = Imstat()
    myimstat.baseframe = mymasterdark
    myimstat.make()

    # Verify that the measured dark current is the same as the real current.
    myreq = DPS3000SCI()
    myreq.dark_true = dark_real
    myreq.imstat_masterdark = myimstat
    myreq.make()
    assert myreq.passed, "Requirement not met!"

    myreqfail = DPS3000SCI()
    myreqfail.dark_true = dark_real * 8
    myreqfail.imstat_masterdark = myimstat
    myreqfail.make()
    assert not myreqfail.passed, "Requirement met inadvertently."

    myreq.commit()
